import { UserTopicFactory } from '../../src/core/factories/user-topic.factory';
import { ExperienceFactory } from '../../src/core/factories/experience.factory';
import { SocialMediaFactory } from '../../src/core/factories/social-media.factory';
import { TopicFactory } from '../../src/core/factories/topic.factory';
import { UserFactory } from '../../src/core/factories/user.factory';
import {
  PrismaClient,
  ReportReason,
  Session,
  Topic,
  User,
} from '@prisma/client';
import { AvailabilityFactory } from '../../src/core/factories/availability.factory';
import { reportReasons, topics } from '../../src/core/utils/constants';
import { SessionFactory } from '../../src/core/factories/session.factory';
import { ReviewFactory } from '../../src/core/factories/review.factory';
import { faker } from '@faker-js/faker';

const list = new Array(30).fill('');
const fixedUsers: Partial<User>[] = [
  {
    id: '038042a6-c7c2-42e7-a80e-0552c7eec925',
    name: 'Daniel Delgado',
    cpf: '11061371913',
    password: 'senha123',
    avatar:
      'https://ik.imagekit.io/h1hun7w2m/avatars/038042a6-c7c2-42e7-a80e-0552c7eec925_LfC5hFDV6.png',
    email: 'daniel.delgado9950@gmail.com',
    isAdmin: true,
    isMentor: true,
  },
  {
    id: '02ebdd13-75f6-46b3-8924-4e58cbab73a1',
    name: 'Adriel Bento',
    cpf: '08093140964',
    password: '08093140964',
    email: 'adrielbb15@gmail.com',
    avatar: 'https://avatars.githubusercontent.com/adrielbento',
    isAdmin: true,
    isMentor: true,
  },
  {
    id: '60ce4f77-faa4-4674-a0b9-a708ea7621af',
    name: 'Gustavo Lara',
    cpf: '08093140926',
    password: '08093140926',
    email: 'gulara19@gmail.com',
    avatar: 'https://avatars.githubusercontent.com/gustavolara',
    isAdmin: true,
    isMentor: true,
  },
];

export async function seedUsers(prisma: PrismaClient): Promise<User[]> {
  console.log('🙍 Seeding users...');

  const adminUsers = fixedUsers.map(async (user: Partial<User>) => {
    return await prisma.user.create({ data: await UserFactory.build(user) });
  });

  const promises = list.map(async () => {
    return await prisma.user.create({ data: await UserFactory.build() });
  });

  const resolvedAdminUsers = await Promise.all(adminUsers);
  const resolvedUsers = await Promise.all(promises);

  return [...resolvedAdminUsers, ...resolvedUsers];
}

export async function seedSocialMedia(prisma: PrismaClient, users: User[]) {
  console.log('📱 Seeding social media...');
  for (let i = 0; i < users.length; i++) {
    for (let j = 0; j < 3; j++) {
      await prisma.socialMedia.create({
        data: SocialMediaFactory.build({ userId: users[i].id }),
      });
    }
  }
}

export async function seedTopic(prisma: PrismaClient) {
  console.log('📚 Seeding topics...');

  const promises = topics.map(async (topic: string) => {
    return await prisma.topic.create({
      data: TopicFactory.build({ name: topic }),
    });
  });

  return await Promise.all(promises);
}

export async function seedExperience(prisma: PrismaClient, users: User[]) {
  console.log('🧾 Seeding experiences...');
  for (let i = 0; i < users.length; i++) {
    for (let j = 0; j < 3; j++) {
      await prisma.experience.create({
        data: ExperienceFactory.build({ userId: users[i].id }),
      });
    }
  }
}

export async function seedUserTopic(
  prisma: PrismaClient,
  users: User[],
  topics: Topic[],
) {
  console.log('🙍📚 Seeding user topics...');
  for (let i = 0; i < users.length; i++) {
    await prisma.userTopic.create({
      data: await UserTopicFactory.build({
        userId: users[i].id,
        topicId: topics[i].id,
      }),
    });
  }
}

export async function seedAvailability(prisma: PrismaClient, users: User[]) {
  console.log('📅 Seeding availability...');

  for (let i = 0; i < users.length; i++) {
    for (let j = 0; j < 7; j++) {
      await prisma.availability.createMany({
        data: await AvailabilityFactory.buildWithService({
          userId: users[i].id,
        }),
        skipDuplicates: true,
      });
    }
  }
}

export async function seedSession(prisma: PrismaClient, users: User[]) {
  console.log('📅 Seeding Session...');
  const sessions: Session[] = [];
  const currentDate = new Date();
  const sessionsPerUser = users.length / 2;
  // const nextMonth = new Date(currentDate.setMonth(currentDate.getMonth() + 1));

  for (let i = 0; i < users.length; i++) {
    for (let j = 0; j < sessionsPerUser; j++) {
      if (i !== j) {
        const nextMonth = new Date(
          currentDate.setMonth(currentDate.getMonth() - 1),
        );

        const data = await SessionFactory.build(
          {
            studentId: users[j].id,
            mentorId: users[i].id,
            startDate: faker.date.recent({
              refDate: nextMonth.toISOString(),
            }),
            endDate: faker.date.future({
              refDate: nextMonth.toISOString(),
            }),
          },
          prisma,
        );

        const session = await prisma.session.create({ data });
        sessions.push(session);
      }
    }
  }

  return sessions;
}
export async function seedPayments(prisma: PrismaClient, sessions: Session[]) {
  console.log('💰 Seeding Payments...');

  for (let i = 0; i < sessions.length; i++) {
    const session = sessions[i];
    const user: User | null = await prisma.user.findUnique({
      where: { id: session.studentId },
    });

    let card = await prisma.paymentCard.findFirst({
      where: { userId: session.studentId },
    });

    if (!card) {
      card = await prisma.paymentCard.create({
        data: {
          userId: session.studentId,
          number: '1234567890123456',
          cvv: '123',
          dueDate: '12/2025',
          name: user?.name || '',
          type: 'credit',
        },
      });
    }

    await prisma.paymentTransaction.create({
      data: {
        value: session.totalValue,
        sessionId: session.id,
        paymentCardId: card.id,
      },
    });
  }
}

export async function seedReportReasons(prisma: PrismaClient) {
  console.log('💬 Seeding Report Reasons...');
  const data: ReportReason[] = [];

  for (let i = 0; i < reportReasons.length; i++) {
    const r = await prisma.reportReason.create({
      data: {
        description: reportReasons[i],
      },
    });

    data.push(r);
  }

  return data;
}

export async function seedReport(
  prisma: PrismaClient,
  users: User[],
  reasons: ReportReason[],
) {
  console.log('🚫 Seeding Report...');

  for (let i = 0; i < users.length; i++) {
    const reportedUserId =
      i === users.length - 1 ? users[0].id : users[i + 1].id;

    const { id: reportReasonId }: ReportReason =
      reasons[Math.floor(Math.random() * reasons.length)];

    await prisma.report.create({
      data: {
        reportingUserId: users[i].id,
        description: faker.lorem.paragraph(),
        reportReasonId,
        reportedUserId,
      },
    });
  }
}

export async function seedReview(prisma: PrismaClient, users: User[]) {
  console.log('🌟 Seeding Review...');

  // for (let i = 0; i < users.length; i++) {
  //   const mentorIds = users
  //     .filter((user) => user.id !== users[i].id)
  //     .map((user) => user.id);

  //   for (let j = 0; j < mentorIds.length; j += 3) {
  //     const mentorId = mentorIds.slice(j, j + 3);

  //     const data = await ReviewFactory.build({
  //       studentId: users[i].id,
  //       mentorId,
  //     });

  //     await prisma.review.create({ data });
  //   }
  // }

  for (let i = 0; i < users.length; i++) {
    const mentorId = i === users.length - 1 ? users[0].id : users[i + 1].id;
    const data = await ReviewFactory.build({
      studentId: users[i].id,
      mentorId,
    });

    await prisma.review.create({ data });
  }
}

export async function seedBannedAccount(prisma: PrismaClient, users: User[]) {
  console.log('🚫 Seeding Banned Account...');
  const bannedUsers = users.slice(10, 16);

  for (let i = 0; i < bannedUsers.length; i++) {
    await prisma.bannedAccount.create({
      data: {
        userId: bannedUsers[i].id,
        reason: faker.lorem.paragraph(),
        startDate: faker.date.past(),
        endDate: faker.date.future(),
        permanently: faker.datatype.boolean(),
      },
    });
  }
}
