import { Module } from '@nestjs/common';
import { AppController } from './web/app.controller';
import { PrismaService } from './core/database/prisma.service';
import { ConfigModule } from '@nestjs/config';
import { KnexModule } from 'nest-knexjs';
import { envSchema, Env } from './env';

import { UserController } from './web/user/user.controller';
import { TopicController } from './web/topic/topic.controller';
import { ConfigService } from '@nestjs/config';

import { UserRepository } from './core/user/user-repository';
import { TopicRepository } from './core/topic/topic-repository';
import { SessionController } from './web/user/session/session.controller';
import { SessionRepository } from './core/session/session-repository';
import { AvailabilityController } from './web/availability/availability.controller';
import { AvailabilityRepository } from './core/availability/availability-repository';
import { UserTopicController } from './web/user/user-topic.controller';
import { UserTopicRepository } from './core/user/user-topic-repository';
import { AuthService } from './core/auth/auth.service';
import { AuthController } from './web/user/auth.controller';
import { MailerModule } from '@nestjs-modules/mailer';
import { PugAdapter } from '@nestjs-modules/mailer/dist/adapters/pug.adapter';
import { SessionService } from './core/session/session.service';
import { PaymentRepository } from './core/session/payment-repository';
import { AvailabilityService } from './core/availability/availability.service';
import { UserService } from './core/user/user.service';
import { StreamChatService } from './core/chat/stream-chat.service';
import { ChatController } from './web/chat/chat.controller';
import { PaymentController } from './web/user/session/payment.controller';
import { UserReportController } from './web/user/report.controller';
import { UserReportRepository } from './core/user/report/user-report-repository';
import { ExperiencesController } from './web/experience/experience.controller';
import { ExperienceRepository } from './core/experience/experience-repository';
import { DashboardRepository } from './core/dashboard/dashboard.repository';
import { DashboardController } from './web/user/dashboard.controller';
import { DashboardService } from './core/dashboard/dashboard.service';
import { SocialMediaController } from './web/user/social-media.controller';
import { SocialMediaRepository } from './core/social-media/social-media-repository';
import { AdminReportsRepository } from './core/admin-reports/admin-reports.repository';
import { AdminReportsController } from './web/admin-reports/admin-reports.controller';
import { AdminReportsService } from './core/admin-reports/admin-reports.service';

@Module({
  imports: [
    ConfigModule.forRoot({
      validate: (env) => envSchema.parse(env),
      isGlobal: true,
    }),
    KnexModule.forRootAsync({
      useFactory: () => {
        const configService = new ConfigService<Env, true>();
        const version = configService.get<Env>('VERSION', { infer: true });
        const host = configService.get<Env>('DB_HOST', { infer: true });
        const user = configService.get<Env>('DB_USER', { infer: true });
        const password = configService.get<Env>('DB_PASSWORD', { infer: true });
        const database = configService.get<Env>('DATABASE', { infer: true });

        return {
          config: {
            client: 'pg',
            version,
            connection: {
              host,
              user,
              password,
              database,
            },
          },
        };
      },
    }),
    MailerModule.forRoot({
      transport: 'smtps://user@domain.com:pass@smtp.domain.com',
      defaults: {
        from: '"nest-modules" <modules@nestjs.com>',
      },
      preview: true,
      template: {
        dir: __dirname + '/core/email/templates',
        adapter: new PugAdapter(),
        options: {
          strict: true,
        },
      },
    }),
  ],
  controllers: [
    AppController,
    UserController,
    TopicController,
    SessionController,
    AvailabilityController,
    UserTopicController,
    AuthController,
    ChatController,
    PaymentController,
    UserReportController,
    ExperiencesController,
    DashboardController,
    SocialMediaController,
    AdminReportsController,
  ],
  providers: [
    // Services
    PrismaService,
    AuthService,
    SessionService,
    AvailabilityService,
    UserService,
    StreamChatService,
    DashboardService,
    AdminReportsService,
    // Repositories
    UserRepository,
    TopicRepository,
    SessionRepository,
    AvailabilityRepository,
    UserTopicRepository,
    SessionRepository,
    PaymentRepository,
    UserReportRepository,
    ExperienceRepository,
    DashboardRepository,
    SocialMediaRepository,
    AdminReportsRepository,
  ],
})
export class AppModule {}
