import { Knex } from 'knex';
import { PrismaService } from '../database/prisma.service';
import { Injectable } from '@nestjs/common';
import { InjectConnection } from 'nest-knexjs';

@Injectable()
export class AdminReportsRepository {
  constructor(
    private prisma: PrismaService,
    @InjectConnection() private readonly knex: Knex,
  ) {}

  async reportMentors() {
    const query = `
        with mentors as  (
            select tu.id, tu.name, tu.score from "tbUser" tu
            left join "tbBannedAccount" tba on tu.id = tba."userId"
            where 
                tu."isAdmin" = false 
                and tu."isMentor" = true
        --		and tba.id = null
        ), denuncias as (
            select tr."reportedUserId" as user_id, COUNT(tr."reportedUserId") as count_report from "tbReport" tr
            group by tr."reportedUserId"
        ), review as (
            select r."mentorId" as user_id, COUNT(r."mentorId") as count_review from "tbReview" r
            join "tbUser" tu on tu.id = r."mentorId"
            where 
                tu."isAdmin" = false 
                and tu."isMentor" = true
            group by r."mentorId"
        ), sessoes as (
            select ts."mentorId" as user_id, COUNT(ts."mentorId") as count_sessions from "tbSession" ts
            where ts.status = 'completed'
            group by ts."mentorId"
        ), aprendizes as (
            select ts."mentorId" as user_id, COUNT(DISTINCT ts."studentId") as count_students from "tbSession" ts
            where ts.status = 'completed'
            group by ts."mentorId"
        ), faturamento_total as (
            select ts."mentorId" as user_id, SUM(ts."totalValue") as count_total_value from "tbSession" ts
            where ts.status = 'completed'
            group by ts."mentorId"
        )
        select
            m.id AS ID,
            m.name AS Nome,
            m.score AS Nota,
            coalesce(d.count_report, 0) as Denuncias,
            coalesce(r.count_review, 0) as Avaliações,
            coalesce(s.count_sessions, 0) as Sessões,
            coalesce(a.count_students, 0) as Mentorados,
            coalesce(ft.count_total_value, 0) as Faturamento
        from
            mentors m
        left join denuncias d ON m.id = d.user_id
        left join review r on m.id = r.user_id
        left join sessoes s on m.id = s.user_id
        left join aprendizes a on m.id = a.user_id
        left join faturamento_total ft on m.id = ft.user_id
    `;

    const result = await this.knex.raw(query);
    return result.rows;
  }

  async getReportFromMentors() {
    const query = `
        select
            tr."reportReasonId",
            trr.description,
            COUNT(tr."reportReasonId")
        from
            "tbReport" tr
        join "tbReportReason" trr on
            tr."reportReasonId" = trr.id
        group by
            tr."reportReasonId",
            trr.description
    `;

    const result = await this.knex.raw(query);
    return result.rows;
  }

  async getAnnualBilling() {
    const query = `
        WITH months AS (
            SELECT date_trunc('month', generate_series(DATE_TRUNC('year', CURRENT_DATE) - INTERVAL '1 year', DATE_TRUNC('year', CURRENT_DATE) + INTERVAL '1 year' - INTERVAL '1 day', '1 month')) AS month
        ),
        session_data AS (
            SELECT
                DATE_TRUNC('month', ta."endDate") AS session_month,
                SUM(ts."totalValue") AS total_month
            FROM
                "tbSession" ts
            JOIN
                "tbAvailability" ta ON ts."availabilityId" = ta.id
            WHERE
                ta."endDate" >= DATE_TRUNC('year', CURRENT_DATE) - INTERVAL '1 year'
                AND ts.status = 'completed'
            GROUP BY
                DATE_TRUNC('month', ta."endDate")
        )
        SELECT
            TO_CHAR(months.month, 'DD/MM/YYYY') as date,
            COALESCE(session_data.total_month, 0) AS total_month
        FROM
            months
        LEFT JOIN
            session_data ON months.month = session_data.session_month
        ORDER BY
            months.month;
    `;

    const result = await this.knex.raw(query);
    return result.rows;
  }

  async getUserGrowthByMonth() {
    const query = `
        WITH months AS (
            SELECT date_trunc('month', generate_series(DATE_TRUNC('year', CURRENT_DATE) - INTERVAL '1 year', DATE_TRUNC('year', CURRENT_DATE) + INTERVAL '1 year' - INTERVAL '1 day', '1 month')) AS month
        ),
        user_data AS (
            SELECT
                DATE_TRUNC('month', tu."createdAt") AS user_month,
                COUNT(tu.id) AS total_month
            FROM
                "tbUser" tu
            WHERE
                tu."createdAt" >= DATE_TRUNC('year', CURRENT_DATE) - INTERVAL '1 year'
            GROUP BY
                DATE_TRUNC('month', tu."createdAt")
        )
        SELECT
            TO_CHAR(months.month, 'DD/MM/YYYY') as date,
            COALESCE(user_data.total_month, 0) AS total_month
        FROM
            months
        LEFT JOIN
            user_data ON months.month = user_data.user_month
        ORDER BY
            months.month;
    `;

    const result = await this.knex.raw(query);
    return result.rows;
  }

  async getUserTypeMetrics() {
    const query = `
            SELECT
                    tu."isMentor",
                    COUNT(tu.id) AS total,
                    COUNT(tu.id) * 100.0 / SUM(COUNT(tu.id)) OVER() AS percentage
            FROM
                    "tbUser" tu
            GROUP BY
                    tu."isMentor";
    `;

    const result = await this.knex.raw(query);
    return result.rows;
  }
}
