import { Injectable } from '@nestjs/common';
import { PrismaService } from '../database/prisma.service';
import { UserRepository } from '../user/user-repository';
import { StreamChatService } from '../chat/stream-chat.service';
import { User } from '@prisma/client';
import { Password } from './password';

@Injectable()
export class AuthService {
  constructor(
    private readonly prismaService: PrismaService,
    private readonly userRepository: UserRepository,
    private readonly streamChatService: StreamChatService,
  ) {}

  async login(data) {
    const user: User = await this.userRepository.login(data);
    const isBanned = await this.userRepository.isBanned(user.id);

    if (!user) {
      throw new Error('Usuário não encontrado');
    }

    if (isBanned) {
      throw new Error('Usuário banido. Acesso negado.');
    }

    console.log(data.password, user.password, user.salt);

    if (!Password.isValid(data.password, user.password, user.salt)) {
      throw new Error('Senha incorreta');
    }

    const chatToken = await this.streamChatService.connect(user.id);

    return { user: { ...user, token: chatToken } };
  }

  async logout(userId: string) {
    await this.streamChatService.disconnect(userId);
  }

  async forgotPassword(email: string): Promise<string> {
    const user = await this.prismaService.user.findUnique({
      where: { email },
    });

    if (!user) {
      throw new Error('Email não encontrado');
    }

    return await this.generateVerificationToken(user.id);
    // Envie o token por e-mail (não implementado aqui)
  }

  async generateVerificationToken(userId: string): Promise<string> {
    const verificationToken = this.generateRandomToken();

    await this.prismaService.passwordToken.create({
      data: {
        token: verificationToken,
        userId,
        expirationDate: new Date(Date.now() + 1000 * 60 * 60),
      },
    });

    return verificationToken;
  }

  async isTokenValid(token: string, userId: string): Promise<boolean> {
    const verificationToken = await this.prismaService.passwordToken.findFirst({
      where: {
        userId,
        token,
      },
    });

    if (!verificationToken || !/^\d{4}$/.test(token)) {
      return false;
    }

    if (verificationToken.expirationDate <= new Date()) {
      return false;
    }

    return true;
  }

  async resetPassword(newPassword: string, userId: string) {
    await this.prismaService.user.update({
      where: { id: userId },
      data: {
        password: newPassword,
      },
    });

    await this.prismaService.passwordToken.delete({ where: { userId } });
  }

  private generateRandomToken(): string {
    const min = 1000;
    const max = 9999;
    const randomNumber = Math.floor(Math.random() * (max - min + 1)) + min;

    return randomNumber.toString();
  }
}
