import { PrismaService } from '@/core/database/prisma.service';
import { Injectable } from '@nestjs/common';
import { isArray } from 'class-validator';
import { Knex } from 'knex';
import { InjectConnection } from 'nest-knexjs';

@Injectable()
export class AvailabilityRepository {
  constructor(
    private prisma: PrismaService,
    @InjectConnection() private readonly knex: Knex,
  ) {}

  async create(data) {
    if (isArray(data)) {
      return await this.prisma.availability.createMany({
        data,
        skipDuplicates: true,
      });
    }

    return await this.prisma.availability.create({
      data: {
        startDate: data.startDate,
        endDate: data.endDate,
        weekDay: data.weekDay,
        userId: data.userId,
      },
    });
  }

  async findById(id: string) {
    const availability = await this.prisma.availability.findUnique({
      where: { id },
      include: { user: true },
    });

    return availability;
  }

  async findByWeekDay(weekDay: string) {
    return await this.prisma.availability.findMany({
      where: { weekDay },
    });
  }

  async listWeekDaysByUserId(userId: string) {
    return await this.prisma.availability.groupBy({
      where: { userId },
      by: ['weekDay'],
    });
  }

  async listByDateAndUserId(date: string, userId: string) {
    const result = await this.knex.raw(
      `
      SELECT 
        s.id as scheduled, a.*
      FROM
      "tbAvailability" a
      LEFT JOIN "tbSession" s on
        a.id = s."availabilityId"
      WHERE
        a."userId" = ?
        AND DATE(a."startDate") = ?
    `,
      [userId, date],
    );

    return result.rows;
  }

  async listByUserToProfileView(userId: string) {
    const result = await this.knex.raw(
      `
      SELECT
        a."weekDay",
        TO_CHAR(a."startDate", 'HH24:MI') as "startDate",
        TO_CHAR(a."endDate", 'HH24:MI') as "endDate"
      FROM
        "tbAvailability" a
      WHERE
        a."userId" = ?
        AND a."disabled" = false
      GROUP BY
        "weekDay",
        TO_CHAR(a."startDate", 'HH24:MI'),
        TO_CHAR(a."endDate", 'HH24:MI')
    `,
      [userId],
    );

    return result.rows;
  }

  async hasAvailability({
    userId,
    weekDay,
    startDate,
    endDate,
  }): Promise<boolean> {
    const query = await this.knex('tbAvailability')
      .count('*')
      .where('userId', userId)
      .where('weekDay', weekDay)
      .where('disabled', true)
      .whereRaw(`TO_CHAR("startDate", 'HH24:MI') = ?`, [startDate])
      .whereRaw(`TO_CHAR("endDate", 'HH24:MI') = ?`, [endDate]);

    return Number(query[0].count) > 0;
  }

  async updateById(id: string, data) {
    return await this.prisma.availability.update({
      where: { id },
      data,
    });
  }

  async delete({ userId, weekDay, startDate, endDate }) {
    return await this.knex('tbAvailability')
      .where(`userId`, userId)
      .where(`weekDay`, weekDay)
      .whereRaw(`TO_CHAR("startDate", 'HH24:MI') = ?`, [startDate])
      .whereRaw(`TO_CHAR("endDate", 'HH24:MI') = ?`, [endDate])
      .update({ disabled: true });
  }

  async deleteById(id: string) {
    return await this.prisma.availability.delete({
      where: { id },
    });
  }
}
