import { Injectable } from '@nestjs/common';
import { Knex } from 'knex';
import { InjectConnection } from 'nest-knexjs';
import { PrismaService } from '../database/prisma.service';

@Injectable()
export class DashboardRepository {
  constructor(
    private readonly prisma: PrismaService,
    @InjectConnection() private readonly knex: Knex,
  ) {}

  async getSessionMetrics(userId: string) {
    const query = `
        WITH sessoes_recentes AS (
            SELECT COUNT(*) AS total_recente
            FROM "tbSession" ts
            JOIN "tbAvailability" ta on	ts."availabilityId" = ta.id
            WHERE ta."endDate" >= CURRENT_DATE - INTERVAL '30 days'
            AND ts.status = 'completed'
            AND ts."mentorId" = '${userId}'
        ),
        sessoes_anteriores AS (
            SELECT COUNT(*) AS total_anterior
            FROM "tbSession" ts
            JOIN "tbAvailability" ta on	ts."availabilityId" = ta.id
            WHERE ta."endDate" < CURRENT_DATE - INTERVAL '30 days'
            AND ta."endDate" >= CURRENT_DATE - INTERVAL '60 days'
            AND ts.status = 'completed'
            AND ts."mentorId" = '${userId}'
        ),
        count_per_day as (
            SELECT
                TO_CHAR(ta."endDate", 'DD/MM/YYYY') as session_date,
                count(*) as per_day
            FROM
                "tbSession" ts
            JOIN "tbAvailability" ta on
                ts."availabilityId" = ta.id
            where
                ta."endDate" >= CURRENT_DATE - interval '30 days'
                AND ts."status" = 'completed'
                AND ts."mentorId" = '${userId}'
            group by
              TO_CHAR(ta."endDate", 'DD/MM/YYYY')
        )
        SELECT
            total_recente as number_sessions_last_30_days,
            total_anterior as number_sessions_last_60_days,
            session_date, per_day,
            CASE
                WHEN total_anterior = 0 THEN 0
                ELSE (total_recente - total_anterior) / CAST(total_anterior AS FLOAT) * 100
            END AS percentage_increase
        FROM sessoes_recentes, sessoes_anteriores, count_per_day;
    `;

    const data = await this.knex.raw(query);
    return data.rows;
  }

  async getSessionRequestsMetrics(userId: string) {
    const query = `
      WITH solicitacoes_recentes AS (
          select COUNT(*) AS total_recente
          FROM "tbSession" ts
          JOIN "tbAvailability" ta on	ts."availabilityId" = ta.id
          WHERE 
            ta."endDate" >= CURRENT_DATE - INTERVAL '30 days'
            AND ts."mentorId" = '${userId}'
      ),
      solicitacoes_anteriores AS (
          SELECT COUNT(*) AS total_anterior
          FROM "tbSession" ts
          JOIN "tbAvailability" ta on	ts."availabilityId" = ta.id
          WHERE ta."endDate" < CURRENT_DATE - INTERVAL '30 days'
            AND ta."endDate" >= CURRENT_DATE - INTERVAL '60 days'
            AND ts."mentorId" = '${userId}'
      ),
      count_per_day as (
        SELECT
          TO_CHAR(ta."endDate", 'DD/MM/YYYY') as session_date,
          count(*) as per_day
        FROM
          "tbSession" ts
        JOIN "tbAvailability" ta on
          ts."availabilityId" = ta.id
        where
          ta."endDate" >= CURRENT_DATE - interval '30 days'
          AND ts."mentorId" = '${userId}'
        group by
          TO_CHAR(ta."endDate", 'DD/MM/YYYY')
      )
      SELECT
          total_recente as number_sessions_last_30_days,
          total_anterior as number_sessions_last_60_days,
          session_date, per_day,
          CASE
              WHEN total_anterior = 0 THEN 0
              ELSE (total_recente - total_anterior) / CAST(total_anterior AS FLOAT) * 100
          END AS percentage_increase
      FROM solicitacoes_recentes, solicitacoes_anteriores, count_per_day
    `;

    const data = await this.knex.raw(query);
    return data.rows;
  }

  async getUsersMetrics(userId: string) {
    const query = `
        WITH aprendizes_recentes AS (
          select COUNT(DISTINCT ts."studentId") AS total_recente
          FROM "tbSession" ts
          JOIN "tbAvailability" ta on	ts."availabilityId" = ta.id
          WHERE 
            ta."endDate" >= CURRENT_DATE - INTERVAL '30 days'
            AND ts.status = 'completed'
            AND ts."mentorId" = '${userId}'
      ),
      aprendizes_anteriores AS (
          SELECT COUNT(DISTINCT ts."studentId") AS total_anterior
          FROM "tbSession" ts
          JOIN "tbAvailability" ta on	ts."availabilityId" = ta.id
          WHERE ta."endDate" < CURRENT_DATE - INTERVAL '30 days'
            AND ta."endDate" >= CURRENT_DATE - INTERVAL '60 days'
            AND ts.status = 'completed'
            AND ts."mentorId" = '${userId}'
      ),
      count_per_day as (
        SELECT
          TO_CHAR(ta."endDate", 'DD/MM/YYYY') as session_date,
          COUNT(DISTINCT ts."studentId") as per_day
        FROM
          "tbSession" ts
        JOIN "tbAvailability" ta on
          ts."availabilityId" = ta.id
        where
          ta."endDate" >= CURRENT_DATE - interval '30 days'
      		AND ts."status" = 'completed'
          AND ts."mentorId" = '${userId}'
        group by
          TO_CHAR(ta."endDate", 'DD/MM/YYYY')
      )
      SELECT
          total_recente as number_sessions_last_30_days,
          total_anterior as number_sessions_last_60_days,
          session_date, per_day,
          CASE
              WHEN total_anterior = 0 THEN 0
              ELSE (total_recente - total_anterior) / CAST(total_anterior AS FLOAT) * 100
          END AS percentage_increase
      FROM aprendizes_recentes, aprendizes_anteriores, count_per_day;
    `;

    const data = await this.knex.raw(query);
    return data.rows;
  }

  async getFinancialMetrics(userId: string) {
    const query = `
      WITH months AS (
        SELECT date_trunc('month', generate_series(DATE_TRUNC('year', CURRENT_DATE) - INTERVAL '1 year', DATE_TRUNC('year', CURRENT_DATE) + INTERVAL '1 year' - INTERVAL '1 day', '1 month')) AS month
      ),
      session_data AS (
          SELECT
              DATE_TRUNC('month', ta."endDate") AS session_month,
              SUM(ts."totalValue") AS total_month
          FROM
              "tbSession" ts
          JOIN
              "tbAvailability" ta ON ts."availabilityId" = ta.id
          WHERE
              ta."endDate" >= DATE_TRUNC('year', CURRENT_DATE) - INTERVAL '1 year'
              AND ts.status = 'completed'
              AND ts."mentorId" = '${userId}'
          GROUP BY
              DATE_TRUNC('month', ta."endDate")
      )
      SELECT
          TO_CHAR(months.month, 'DD/MM/YYYY') as date,
          COALESCE(session_data.total_month, 0) AS total_month
      FROM
          months
      LEFT JOIN
          session_data ON months.month = session_data.session_month
      ORDER BY
          months.month
    `;

    const data = await this.knex.raw(query);
    return data.rows;
  }
}
