import { Injectable } from '@nestjs/common';
import { Session, UserTopic, Availability } from '.prisma/client';
import { faker } from '@faker-js/faker';
import { PrismaClient } from '@prisma/client';
import { UserTopicFactory } from './user-topic.factory';
import { AvailabilityFactory } from './availability.factory';
import { SessionStatus } from '../types';

@Injectable()
export class SessionFactory {
  static async create(
    prisma: PrismaClient,
    overrides: Partial<Session> = {},
  ): Promise<Session> {
    const session = await SessionFactory.build(overrides, prisma);
    return await prisma.session.create({
      data: session,
    });
  }

  static async build(
    overrides: Partial<Session> = {},
    prisma: PrismaClient | null = null,
  ) {
    const studentId = overrides.studentId || faker.string.uuid();
    const mentorId = overrides.mentorId || faker.string.uuid();
    let userTopic = {} as UserTopic;
    let availability = {} as Availability;
    let cancelledReason: string = '';

    if (prisma) {
      userTopic = await SessionFactory.createUserTopic(prisma, mentorId);
      availability = await SessionFactory.createAvailability(prisma, mentorId);
    }

    const statusValues = Object.values(SessionStatus);
    const randomStatus =
      statusValues[Math.floor(Math.random() * statusValues.length)];

    if (randomStatus === SessionStatus.CANCELLED) {
      cancelledReason = faker.lorem.sentence();
    }

    return {
      id: faker.string.uuid(),
      startDate: overrides.startDate || faker.date.recent(),
      endDate: overrides.endDate || faker.date.future(),
      status: randomStatus,
      description: faker.lorem.paragraph(),
      cancelledReason,
      totalValue: faker.number.float({ min: 0, max: 500, precision: 0.01 }),
      userTopicId: userTopic?.id,
      studentId,
      mentorId,
      availabilityId: availability?.id,
      ...overrides,
    };
  }

  private static async createUserTopic(prisma: PrismaClient, userId: string) {
    return await prisma.userTopic.create({
      data: await UserTopicFactory.build({ userId }, prisma),
    });
  }

  private static async createAvailability(
    prisma: PrismaClient,
    userId: string,
  ) {
    return await prisma.availability.create({
      data: await AvailabilityFactory.build({ userId }, prisma),
    });
  }
}
