import { Injectable } from '@nestjs/common';
import { SessionRepository } from './session-repository';
import { PaymentTransactionStatus, SessionStatus } from '../types';
import { PaymentRepository } from './payment-repository';
import { Session } from '@prisma/client';

@Injectable()
export class SessionService {
  constructor(
    private readonly sessionRepository: SessionRepository,
    private readonly paymentRepository: PaymentRepository,
  ) {}

  async create(session) {
    const { paymentCardId } = session;
    delete session.paymentCardId;

    const data: Partial<Session> = {
      ...session,
      startDate: new Date(session.startDate),
      endDate: new Date(session.endDate),
    };

    const createdSession: Session = await this.sessionRepository.create(data);
    const transaction = {
      value: createdSession.totalValue,
      sessionId: createdSession.id,
      paymentCardId,
    };

    return await this.paymentRepository.createTransaction(transaction);
  }

  async accept(sessionId: string) {
    await this.sessionRepository.update(sessionId, {
      status: SessionStatus.CONFIRMED,
    });

    await this.paymentRepository.updateTransaction(sessionId, {
      status: PaymentTransactionStatus.PAID,
    });
  }

  async decline(sessionId: string) {
    await this.sessionRepository.update(sessionId, {
      status: SessionStatus.DECLINED,
    });

    await this.paymentRepository.updateTransaction(sessionId, {
      status: PaymentTransactionStatus.CANCELLED,
    });
  }

  async cancel(sessionId: string, reason: string) {
    await this.sessionRepository.update(sessionId, {
      status: SessionStatus.CANCELLED,
      cancelledReason: reason,
    });

    await this.paymentRepository.updateTransaction(sessionId, {
      status: PaymentTransactionStatus.CANCELLED,
    });
  }

  async listByUserId(userId: string) {
    const sessions = await this.sessionRepository.listByUserId(userId);

    return sessions.map((session) => {
      return {
        ...session,
        asMentor: session.mentorId === userId,
        avatar:
          session.mentorId === userId
            ? session.student.avatar
            : session.mentor.avatar,
        name:
          session.mentorId === userId
            ? session.student.name
            : session.mentor.name,
        studentOrMentorId:
          session.mentorId === userId ? session.studentId : session.mentorId,
      };
    });
  }
}
