import { Injectable } from '@nestjs/common';
import { PrismaService } from '@/core/database/prisma.service';
import { SessionStatus } from '../types';
import { Knex } from 'knex';
import { InjectConnection } from 'nest-knexjs';

@Injectable()
export class UserRepository {
  constructor(
    private prisma: PrismaService,
    @InjectConnection() private readonly knex: Knex,
  ) {}

  async getAll() {
    return await this.prisma.user.findMany();
  }

  async create(data) {
    return await this.prisma.user.create({
      data: {
        name: data.name,
        cpf: data.cpf,
        email: data.email,
        password: data.password,
        salt: data.salt,
        avatar: data.avatar,
        abstract: data.abstract,
        isAdmin: data.isAdmin,
        isMentor: data.isMentor,
      },
    });
  }

  async getById(id: string) {
    return await this.prisma.user.findUnique({
      where: {
        id: id,
      },
      include: {
        experiences: true,
        socialMedias: true,
      },
    });
  }

  async getByIdToMentorView(id: string) {
    return await this.prisma.user.findUnique({
      where: {
        id: id,
      },
      include: {
        experiences: true,
        socialMedias: {
          select: {
            type: true,
            url: true,
          },
        },
        userTopics: {
          select: {
            topic: {
              select: {
                name: true,
              },
            },
          },
        },
        reviewToMentor: {
          select: {
            grade: true,
            evaluation: true,
            createdAt: true,
            student: {
              select: {
                name: true,
                avatar: true,
                id: true,
              },
            },
          },
        },
      },
    });
  }

  async getAdmins() {
    return await this.prisma.user.findMany({
      where: {
        isAdmin: true,
      },
      include: {
        experiences: true,
      },
    });
  }

  async updateById(id, data) {
    return await this.prisma.user.update({
      where: {
        id: id,
      },
      data: {
        name: data.name,
        cpf: data.cpf,
        email: data.email,
        password: data.password,
        avatar: data.avatar,
        abstract: data.abstract,
        isAdmin: data.isAdmin,
      },
    });
  }

  async deleteUserById(id) {
    return await this.prisma.user.delete({
      where: {
        id: id,
      },
    });
  }

  async login({ cpf }) {
    return await this.prisma.user.findFirstOrThrow({
      where: { cpf },
    });
  }

  async reviewUser(userId, body) {
    return await this.prisma.review.create({
      data: { studentId: userId, ...body },
    });
  }

  async report(body) {
    return await this.prisma.report.create({
      data: body,
    });
  }

  async banUser(data) {
    return await this.prisma.bannedAccount.create({
      data,
    });
  }

  async isBanned(userId: string) {
    return await this.prisma.bannedAccount.findFirst({
      where: {
        userId,
      },
    });
  }

  async listMentors() {
    return await this.prisma.user.findMany({
      where: {
        isMentor: true,
      },
      include: {
        userTopics: {
          select: {
            topic: {
              select: {
                name: true,
                id: true,
              },
            },
          },
        },
      },
    });
  }

  async listReviewsByMentorId(mentorId: string) {
    return await this.prisma.review.findMany({
      where: {
        mentorId,
      },
      include: {
        student: {
          select: {
            name: true,
            avatar: true,
          },
        },
      },
    });
  }

  async getAvgGradeReviewByMentorId(mentorId: string) {
    return await this.prisma.review.aggregate({
      where: {
        mentorId,
      },
      _avg: {
        grade: true,
      },
    });
  }

  async getMentorHomePageData(mentorId: string) {
    // 1. Retornar quantidade de mentorias realizadas

    const totalSessions = await this.prisma.session.aggregate({
      where: {
        mentorId,
        status: SessionStatus.COMPLETED,
      },
      _count: {
        id: true,
      },
    });

    // Retorna as próximas mentorias agendadas
    const queryNextSessions = `
      SELECT
        tu.id AS id,
        tu."name" AS name,
        ta."startDate",
        ta."endDate",
        TO_CHAR(ta."startDate", 'HH12:MI AM') || ' até ' || TO_CHAR(ta."endDate", 'HH12:MI AM') AS session_time,
        '' AS rest_time
      FROM
          "tbSession" ts
      JOIN
          "tbUser" tu ON tu.id = ts."studentId"
      JOIN
          "tbAvailability" ta ON ta.id = ts."availabilityId"
      WHERE
          ts."mentorId" = '${mentorId}'
          AND ts.status = '${SessionStatus.CONFIRMED}'
    `;

    const { rows: nextScheduledSessions } =
      await this.knex.raw(queryNextSessions);

    const queryStudents = `
      SELECT
          ts."studentId" as id,
          tu.name,
          COUNT(ts.id) as total_sessions
      FROM
          "tbSession" ts
      JOIN
          "tbUser" tu ON tu.id = ts."studentId"
      WHERE
          "mentorId" = '${mentorId}'
          AND status = '${SessionStatus.CONFIRMED}'
      GROUP BY
          ts."studentId",
          tu.name;
    `;

    const { rows: students } = await this.knex.raw(queryStudents);

    return {
      totalSessions: totalSessions._count.id,
      nextScheduledSessions,
      users: students,
    };
  }
}
