import { Injectable } from '@nestjs/common';
import { UserRepository } from './user-repository';
import { StreamChatService } from '../chat/stream-chat.service';
import { User } from '@prisma/client';
import ImageKit from 'imagekit';
import { ConfigService } from '@nestjs/config';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import type { Multer } from 'multer';
import { Password } from '../auth/password';

@Injectable()
export class UserService {
  private readonly imagekit: ImageKit;

  constructor(
    private readonly userRepository: UserRepository,
    private readonly streamChatService: StreamChatService,
    private readonly configService: ConfigService,
  ) {
    this.imagekit = new ImageKit({
      publicKey: this.configService.get<string>('IMAGEKIT_PUBLIC_KEY')!,
      privateKey: this.configService.get<string>('IMAGEKIT_PRIVATE_KEY')!,
      urlEndpoint: this.configService.get<string>('IMAGEKIT_URL_ENDPOINT')!,
    });
  }

  async create(data: any): Promise<User> {
    const [salt, password] = await Password.hashPassword(data.password);

    const user: User = await this.userRepository.create({
      ...data,
      password,
      salt,
    });

    await this.streamChatService.createUser(user.id, user.name);

    return user;
  }

  async reviewUser(studentId: string, body: any) {
    const review = await this.userRepository.reviewUser(studentId, body);

    const averageScore = await this.userRepository.getAvgGradeReviewByMentorId(
      body?.mentorId,
    );

    await this.userRepository.updateById(studentId, { score: averageScore });

    return review;
  }

  async banUser(userId: string, data) {
    return await this.userRepository.banUser({ ...data, userId });
  }

  async getMentorHomePageData(mentorId: string) {
    return await this.userRepository.getMentorHomePageData(mentorId);
  }

  async uploadAvatarImage(
    id: string,
    body: { file: string },
    fileFormData: Express.Multer.File,
  ) {
    const fileBase64 = fileFormData?.buffer || body.file;

    const result = await this.imagekit.upload({
      file: fileBase64,
      fileName: `${id}.png`,
      folder: 'avatars',
    });

    const { name }: User = await this.userRepository.updateById(id, {
      avatar: result.url,
    });

    await this.streamChatService.updateUser(id, name, result.url);

    return { url: result.url };
  }
}
