import { AvailabilityRepository } from '@/core/availability/availability-repository';
import {
  BadRequestException,
  Body,
  Controller,
  Delete,
  Get,
  NotFoundException,
  Param,
  Post,
  Put,
  Query,
  UsePipes,
} from '@nestjs/common';
import { z } from 'zod';
import { ZodValidationPipe } from '../pipes/zod-validation-pipe';
import { AvailabilityService } from '@/core/availability/availability.service';

const weekDays = [
  'Segunda',
  'Terça',
  'Quarta',
  'Quinta',
  'Sexta',
  'Sábado',
  'Domingo',
] as const;

const createSchemaAvailability = z.object({
  startDate: z.string(),
  endDate: z.string(),
  weekDay: z.enum(weekDays),
  userId: z.string().uuid(),
});

const updateSchemaAvailability = z.object({
  startDate: z.string(),
  endDate: z.string(),
  weekDay: z.enum(weekDays),
});

type CreateAvailabilityBodySchema = z.infer<typeof createSchemaAvailability>;
type UpdateAvailabilityBodySchema = z.infer<typeof updateSchemaAvailability>;

@Controller('availability')
export class AvailabilityController {
  constructor(
    private readonly availabilityRepository: AvailabilityRepository,
    private readonly availabilityService: AvailabilityService,
  ) {}

  @Get('/search')
  async search(@Query('userId') userId: string, @Query('date') date: string) {
    let availability: any = [];

    if (date) {
      availability = await this.availabilityRepository.listByDateAndUserId(
        date,
        userId,
      );
    }

    if (!availability) {
      throw new NotFoundException('Erro ao buscar disponibilidade', {
        description: 'Disponibilidade não encontrada',
      });
    }

    return availability;
  }

  @Get(':id')
  async getById(@Param('id') id: string) {
    const availability = await this.availabilityRepository.findById(id);

    if (!availability) {
      throw new NotFoundException('Erro ao buscar disponibilidade', {
        description: 'Disponibilidade não encontrada por ID',
      });
    }

    return availability;
  }

  @Get('user/:userId/weekday')
  async listWeekDays(@Param('userId') userId: string) {
    const availability =
      await this.availabilityRepository.listWeekDaysByUserId(userId);

    if (!availability) {
      throw new NotFoundException('Erro ao buscar dias da semana', {
        description: 'Dias da semana não encontrada por UserID',
      });
    }

    return availability;
  }

  @Get('user/:userId')
  async listByUserToProfileView(@Param('userId') userId: string) {
    const availability =
      await this.availabilityRepository.listByUserToProfileView(userId);

    if (!availability) {
      return [];
    }

    return availability;
  }

  @Post()
  @UsePipes(new ZodValidationPipe(createSchemaAvailability))
  async create(@Body() data: CreateAvailabilityBodySchema) {
    try {
      return await this.availabilityService.create(data);
    } catch (error) {
      throw new BadRequestException(
        `Erro ao criar disponibilidade\nError -> ${error.message}`,
        {
          cause: error,
        },
      );
    }
  }

  @Put(':id')
  @UsePipes(new ZodValidationPipe(updateSchemaAvailability))
  async update(
    @Param('id') id: string,
    @Body() data: UpdateAvailabilityBodySchema,
  ) {
    try {
      return await this.availabilityRepository.updateById(id, data);
    } catch (error) {
      throw new BadRequestException('Erro ao atualizar disponibilidade', {
        cause: error,
      });
    }
  }

  @Delete()
  @UsePipes(new ZodValidationPipe(updateSchemaAvailability))
  async delete(
    @Query('userId') userId: string,
    @Body() data: UpdateAvailabilityBodySchema,
  ) {
    try {
      return await this.availabilityService.delete({ userId, ...data });
    } catch (error) {
      throw new BadRequestException(
        `Erro ao deletar disponibilidade\nError -> ${error.message}`,
        {
          cause: error.message,
        },
      );
    }
  }
}
