import {
  Body,
  Controller,
  Delete,
  Get,
  Param,
  Post,
  Put,
  UsePipes,
} from '@nestjs/common';
import { z } from 'zod';
import { ZodValidationPipe } from '../pipes/zod-validation-pipe';
import { ExperienceRepository } from '@/core/experience/experience-repository';

const schemaExperience = z.object({
  id: z.string().uuid().optional(),
  userId: z.string().uuid(),
  company: z.string(),
  role: z.string(),
  description: z.string(),
  startDate: z.coerce.date(),
  endDate: z.coerce.date().optional(),
});

type ExperienceBodySchema = z.infer<typeof schemaExperience>;

@Controller('/experiences')
export class ExperiencesController {
  constructor(private experienceRepository: ExperienceRepository) {}

  @Post()
  @UsePipes(new ZodValidationPipe(schemaExperience))
  async create(@Body() experience: ExperienceBodySchema) {
    return await this.experienceRepository.create(experience);
  }

  @Get(':userId')
  async findAll(@Param('userId') userId: string) {
    return await this.experienceRepository.findAllExperiencesByUserId(userId);
  }

  @Get(':userId/:id')
  async findOne(@Param('userId') userId: string, @Param('id') id: string) {
    return await this.experienceRepository.findExperienceById(id);
  }

  @Put(':userId/:id')
  @UsePipes(new ZodValidationPipe(schemaExperience))
  async update(
    @Param('userId') userId: string,
    @Param('id') id: string,
    @Body() experience: ExperienceBodySchema,
  ) {
    return await this.experienceRepository.updateExperienceById(id, experience);
  }

  @Delete(':userId/:id')
  async delete(@Param('userId') userId: string, @Param('id') id: string) {
    return await this.experienceRepository.deleteExperienceById(id);
  }
}
