import { AuthService } from '@/core/auth/auth.service';
import { PrismaService } from '@/core/database/prisma.service';
import { ZodValidationPipe } from '@/web/pipes/zod-validation-pipe';
import { z } from 'zod';
import {
  Controller,
  Post,
  Body,
  HttpCode,
  Param,
  Put,
  UsePipes,
  NotFoundException,
  BadRequestException,
} from '@nestjs/common';

const schemaValidateToken = z.object({
  token: z.string(),
  email: z.string(),
});

type ValidateTokenBodySchema = z.infer<typeof schemaValidateToken>;

@Controller('auth')
export class AuthController {
  constructor(
    private readonly prismaService: PrismaService,
    private readonly authService: AuthService,
  ) {}

  @Post('login')
  async login(@Body() body) {
    try {
      return await this.authService.login(body);
    } catch (error) {
      throw new BadRequestException('Erro ao realizar login', {
        cause: error,
      });
    }
  }

  @Post('logout/:userId')
  async logout(@Param('userId') userId: string) {
    try {
      return await this.authService.logout(userId);
    } catch (error) {
      throw new BadRequestException('Erro ao realizar o logOut', {
        cause: error,
      });
    }
  }

  @Post('forgot-password')
  @HttpCode(200)
  async forgotPassword(@Body() body: { email: string }) {
    try {
      const token = await this.authService.forgotPassword(body.email);
      return {
        message: 'E-mail com token de verificação enviado',
        token,
      };
    } catch (error) {
      throw new BadRequestException(
        'Erro ao enviar e-mail de recuperação de senha',
        {
          cause: error,
        },
      );
    }
  }

  @Post('validate-token')
  @HttpCode(200)
  @UsePipes(new ZodValidationPipe(schemaValidateToken))
  async validateToken(@Body() body: ValidateTokenBodySchema) {
    const user = await this.prismaService.user.findUnique({
      where: { email: body.email },
    });

    if (!user) throw new NotFoundException('Usuário não encontrado');

    const isValid = await this.authService.isTokenValid(body.token, user.id);

    if (isValid) return { message: 'Token válido', userId: user.id };
    throw new BadRequestException('Token inválido');
  }

  @Put('redefine-password/:userId')
  async redefinePassword(@Param('userId') userId: string, @Body() body: any) {
    try {
      await this.authService.resetPassword(body?.newPassword, userId);
    } catch (error) {
      throw new BadRequestException('Erro ao redefinir senha', {
        cause: error,
      });
    }
  }
}
