import {
  BadRequestException,
  Body,
  Controller,
  Delete,
  Get,
  Param,
  Post,
  Put,
  UsePipes,
} from '@nestjs/common';
import { z } from 'zod';
import { ZodValidationPipe } from '../../pipes/zod-validation-pipe';
import { SessionRepository } from '@/core/session/session-repository';
import { SessionService } from '@/core/session/session.service';
import { SessionStatus } from '@/core/types';

const schemaSession = z.object({
  id: z.string().optional(),
  startDate: z.coerce.date(),
  endDate: z.coerce.date(),
  status: z
    .enum([
      SessionStatus.PENDING,
      SessionStatus.CONFIRMED,
      SessionStatus.CANCELLED,
      SessionStatus.COMPLETED,
    ])
    .optional(),
  cancelledReason: z.string().optional(),
  userTopicId: z.string(),
  studentId: z.string(),
  mentorId: z.string(),
  toCombine: z.boolean().optional(),
  combinedHours: z.number().optional(),
  paymentCardId: z.string(),
  description: z.string(),
  totalValue: z.number(),
  availabilityId: z.string().optional(),
});

type SessionBodySchema = z.infer<typeof schemaSession>;

@Controller('/sessions')
export class SessionController {
  constructor(
    private sessionRepository: SessionRepository,
    private sessionService: SessionService,
  ) {}

  @Get('user/:id')
  async list(@Param('id') id: string) {
    return await this.sessionService.listByUserId(id);
  }

  @Get(':id')
  async getById(@Param('id') id: string) {
    return await this.sessionRepository.getById(id);
  }

  @Post()
  @UsePipes(new ZodValidationPipe(schemaSession))
  async create(@Body() session: SessionBodySchema) {
    try {
      return await this.sessionService.create(session);
    } catch (error) {
      throw new BadRequestException(
        `Erro ao solicitar uma mentoria -> ${error.message}`,
      );
    }
  }

  @Put(':id')
  @UsePipes(new ZodValidationPipe(schemaSession))
  async update(@Param('id') id: string, @Body() session: SessionBodySchema) {
    return await this.sessionRepository.update(id, session);
  }

  @Delete(':id')
  async delete(@Param('id') id: string) {
    return await this.sessionRepository.delete(id);
  }

  @Put(':id/accept')
  async accept(@Param('id') id: string) {
    try {
      return await this.sessionService.accept(id);
    } catch (error) {
      throw new BadRequestException(
        `Erro ao aceitar mentoria -> ${error.message}`,
      );
    }
  }

  @Put(':id/decline')
  async decline(@Param('id') id: string) {
    try {
      return await this.sessionService.decline(id);
    } catch (error) {
      throw new BadRequestException(
        `Erro ao Rejeitar mentoria -> ${error.message}`,
      );
    }
  }

  @Put(':id/cancel')
  async cancel(
    @Param('id') id: string,
    @Body() body: { cancelledReason: string },
  ) {
    try {
      return await this.sessionService.cancel(id, body.cancelledReason);
    } catch (error) {
      throw new BadRequestException(
        `Erro ao Cancelar mentoria -> ${error.message}`,
      );
    }
  }
}
