import { UserTopicRepository } from '@/core/user/user-topic-repository';
import { ZodValidationPipe } from '@/web/pipes/zod-validation-pipe';
import { z } from 'zod';

import {
  Controller,
  Get,
  Post,
  Put,
  Delete,
  Param,
  Body,
  UsePipes,
  NotFoundException,
  BadRequestException,
} from '@nestjs/common';

const createSchemaUserTopic = z.object({
  price: z.number().positive(),
  userId: z.string().uuid(),
  topicId: z.string().uuid(),
});

const updateSchemaUserTopic = z.object({
  price: z.number().positive(),
  userId: z.string().uuid(),
  topicId: z.string().uuid(),
});

type CreateUserTopicBodySchema = z.infer<typeof createSchemaUserTopic>;
type UpdateUserTopicBodySchema = z.infer<typeof updateSchemaUserTopic>;

@Controller('user-topics')
export class UserTopicController {
  constructor(private readonly userTopicRepository: UserTopicRepository) {}

  @Get(':id')
  async getById(@Param('id') id: string) {
    const userTopic = await this.userTopicRepository.findById(id);

    if (!userTopic) {
      throw new NotFoundException('Tópico não encontrado');
    }

    return userTopic;
  }

  @Get('user/:userId')
  async getByUserId(@Param('userId') userId: string) {
    const userTopic = await this.userTopicRepository.findByUserId(userId);

    if (!userTopic) {
      throw new NotFoundException('Tópico não encontrado');
    }

    return userTopic;
  }

  @Get('topic/:topicId')
  async getByTopicId(@Param('topicId') topicId: string) {
    const userTopic = await this.userTopicRepository.findByTopicId(topicId);

    if (!userTopic) {
      throw new NotFoundException('Tópico não encontrado');
    }

    return userTopic;
  }

  @Post()
  @UsePipes(new ZodValidationPipe(createSchemaUserTopic))
  async create(@Body() data: CreateUserTopicBodySchema) {
    try {
      return await this.userTopicRepository.create(data);
    } catch (error) {
      throw new BadRequestException('Erro ao atualizar tópico', {
        cause: error,
      });
    }
  }

  @Put(':id')
  @UsePipes(new ZodValidationPipe(updateSchemaUserTopic))
  async update(
    @Param('id') id: string,
    @Body() data: UpdateUserTopicBodySchema,
  ) {
    try {
      return await this.userTopicRepository.updateById(id, data);
    } catch (error) {
      throw new BadRequestException('Erro ao atualizar tópico', {
        cause: error,
      });
    }
  }

  @Delete(':id')
  async delete(@Param('id') id: string) {
    try {
      return await this.userTopicRepository.deleteById(id);
    } catch (error) {
      throw new BadRequestException('Erro ao deletar tópico', {
        cause: error,
      });
    }
  }
}
