import {
  Body,
  Controller,
  Delete,
  Get,
  Param,
  Post,
  Put,
  UploadedFile,
  UseInterceptors,
  UsePipes,
} from '@nestjs/common';
import { z } from 'zod';
import { ZodValidationPipe } from '../pipes/zod-validation-pipe';
import { UserRepository } from '@/core/user/user-repository';
import { UserService } from '@/core/user/user.service';
import { FileInterceptor } from '@nestjs/platform-express';
import { Express } from 'express';
// eslint-disable-next-line @typescript-eslint/no-unused-vars
import type { Multer } from 'multer';

const schemaPassword = z.object({
  password: z.string(),
});

const schemaUser = z.object({
  name: z.string(),
  cpf: z.string(),
  email: z.string().email(),
  avatar: z.string().optional(),
  abstract: z.string().optional(),
  isAdmin: z.boolean().optional(),
  isMentor: z.boolean().optional(),
});

const schemaReview = z.object({
  mentorId: z.string(),
  grade: z.number().min(1).max(5),
  evaluation: z.string().optional(),
});

const schemaReport = z.object({
  reportReasonId: z.string(),
  description: z.string(),
  reportedUserId: z.string(),
});

const schemaBanned = z.object({
  reason: z.string(),
  permanently: z.boolean(),
  startDate: z.string(),
  endDate: z.string(),
});

type ReviewBodySchema = z.infer<typeof schemaReview>;
type ReportBodySchema = z.infer<typeof schemaReport>;
type BanBodySchema = z.infer<typeof schemaBanned>;
type UserBodySchema = z.infer<typeof schemaUser>;
type UserPasswordBodySchema = z.infer<typeof schemaPassword>;

@Controller('/users')
export class UserController {
  constructor(
    private readonly userRepository: UserRepository,
    private readonly userService: UserService,
  ) {}

  @Get()
  async findAll() {
    return await this.userRepository.getAll();
  }

  @Get('mentors')
  async listMentors() {
    return await this.userRepository.listMentors();
  }

  @Get('admins')
  async findAllAdmins() {
    return await this.userRepository.getAdmins();
  }

  @Get(':id')
  async findOne(@Param('id') id: string) {
    return await this.userRepository.getById(id);
  }

  @Get(':id/mentor-view')
  async getUserToMentorView(@Param('id') id: string) {
    return await this.userRepository.getByIdToMentorView(id);
  }

  @Post()
  @UsePipes(new ZodValidationPipe(schemaUser))
  async create(@Body() body: UserBodySchema) {
    return await this.userService.create(body);
  }

  @Put(':id')
  @UsePipes(new ZodValidationPipe(schemaUser))
  async update(@Param('id') id: string, @Body() body: UserBodySchema) {
    return await this.userRepository.updateById(id, body);
  }

  @Put(':id/password')
  @UsePipes(new ZodValidationPipe(schemaPassword))
  async updatePassword(
    @Param('id') id: string,
    @Body() body: UserPasswordBodySchema,
  ) {
    return await this.userRepository.updateById(id, body);
  }

  @Put(':id/avatar')
  @UseInterceptors(FileInterceptor('file'))
  async uploadAvatar(
    @Param('id') id: string,
    @Body() body: any,
    @UploadedFile() file: Express.Multer.File,
  ) {
    return await this.userService.uploadAvatarImage(id, body, file);
  }

  @Delete(':id')
  async delete(@Param('id') id: string) {
    return await this.userRepository.deleteUserById(id);
  }

  @Post(':id/review')
  @UsePipes(new ZodValidationPipe(schemaReview))
  async createReview(
    @Param('id') userId: string,
    @Body() body: ReviewBodySchema,
  ) {
    return await this.userService.reviewUser(userId, body);
  }

  @Post(':id/report')
  @UsePipes(new ZodValidationPipe(schemaReport))
  async report(@Param('id') userId: string, @Body() body: ReportBodySchema) {
    return await this.userRepository.report({
      ...body,
      reportingUserId: userId,
    });
  }

  @Post(':id/ban')
  async banUser(@Param('id') userId: string, @Body() body: BanBodySchema) {
    return await this.userService.banUser(userId, body);
  }

  @Get(':id/home')
  async getMentorHome(@Param('id') id: string) {
    return await this.userService.getMentorHomePageData(id);
  }
}
