import { Test, TestingModule } from '@nestjs/testing';
import { HttpStatus, INestApplication } from '@nestjs/common';
import request from 'supertest';
import { AppModule } from '@/app.module';
import { UserFactory } from '@/core/factories/user.factory';
import { PrismaService } from '@/core/database/prisma.service';
import { AuthService } from '@/core/auth/auth.service';

describe('AuthController (e2e)', () => {
  let app: INestApplication;
  let prisma: PrismaService;
  let auth: AuthService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AppModule],
      providers: [UserFactory],
    }).compile();

    app = module.createNestApplication();
    prisma = module.get<PrismaService>(PrismaService);
    auth = module.get<AuthService>(AuthService);

    await app.init();
  });

  it('should send a forgot password email', async () => {
    const user = await UserFactory.create(prisma);

    const response = await request(app.getHttpServer())
      .post('/auth/forgot-password')
      .send({ email: user.email })
      .expect(HttpStatus.OK);

    expect(response.body).toHaveProperty(
      'message',
      'E-mail com token de verificação enviado',
    );

    const userWithToken = await prisma.passwordToken.findUnique({
      where: { userId: user.id },
    });

    expect(userWithToken?.token).not.toBeNull();
  });

  it('should validate a token', async () => {
    const user = await UserFactory.create(prisma);
    const token = await auth.generateVerificationToken(user.id);

    const response = await request(app.getHttpServer())
      .post('/auth/validate-token')
      .send({ token, email: user.email })
      .expect(HttpStatus.OK);

    expect(response.body).toHaveProperty('message', 'Token válido');
    expect(response.body).toHaveProperty('userId', user.id);
  });

  it('should not validate an invalid token', async () => {
    const user = await UserFactory.create(prisma);
    const token = 'invalid-token';

    await request(app.getHttpServer())
      .post('/auth/validate-token')
      .send({ token, email: user.email })
      .expect(HttpStatus.BAD_REQUEST);
  });

  it('should reset a user password', async () => {
    const user = await UserFactory.create(prisma);
    await auth.generateVerificationToken(user.id);

    const newPassword = 'new-password';

    await request(app.getHttpServer())
      .put(`/auth/redefine-password/${user.id}`)
      .send({ newPassword })
      .expect(HttpStatus.OK);
  });
});
