import { Test, TestingModule } from '@nestjs/testing';
import { HttpStatus, INestApplication } from '@nestjs/common';
import { AppModule } from '@/app.module';
import request from 'supertest';
import { AvailabilityFactory } from '@/core/factories/availability.factory';
import { PrismaService } from '@/core/database/prisma.service';

describe('AvailabilityController (e2e)', () => {
  let app: INestApplication;
  let prisma: PrismaService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AppModule],
      providers: [AvailabilityFactory],
    }).compile();

    app = module.createNestApplication();
    prisma = module.get<PrismaService>(PrismaService);

    await app.init();
  });

  it('should create a new availability', async () => {
    const newAvailability = await AvailabilityFactory.create(prisma);

    const response = await request(app.getHttpServer())
      .post('/availability')
      .send({
        userId: newAvailability.userId,
        startDate: newAvailability.startDate.toISOString(),
        endDate: newAvailability.endDate.toISOString(),
        weekDay: newAvailability.weekDay,
      })
      .expect(HttpStatus.CREATED);

    expect(response.body).toHaveProperty('id');
    expect(response.body.startDate).toEqual(
      newAvailability.startDate.toISOString(),
    );
    expect(response.body.endDate).toEqual(
      newAvailability.endDate.toISOString(),
    );
  });

  it('should get availability by ID', async () => {
    const newAvailability = await AvailabilityFactory.create(prisma);

    const response = await request(app.getHttpServer())
      .get(`/availability/${newAvailability.id}`)
      .expect(HttpStatus.OK);

    expect(response.body.id).toEqual(newAvailability.id);
    expect(response.body.startDate).toEqual(
      newAvailability.startDate.toISOString(),
    );
    expect(response.body.endDate).toEqual(
      newAvailability.endDate.toISOString(),
    );
  });

  it('should get availability by user ID', async () => {
    const newAvailability = await AvailabilityFactory.create(prisma);

    const response = await request(app.getHttpServer())
      .get(`/availability/user/${newAvailability.userId}`)
      .expect(HttpStatus.OK);

    expect(Array.isArray(response.body)).toBeTruthy();
    expect(response.body.length).toBeGreaterThan(0);
    expect(response.body[0].userId).toEqual(newAvailability.userId);
  });

  // it('should update availability', async () => {
  //   const newAvailability = await AvailabilityFactory.create(prisma);
  //   const updatedAvailability = await AvailabilityFactory.build();

  //   const response = await request(app.getHttpServer())
  //     .put(`/availability/${newAvailability.id}`)
  //     .send({
  //       startDate: updatedAvailability.startDate.toISOString(),
  //       endDate: updatedAvailability.endDate.toISOString(),
  //       weekDay: updatedAvailability.weekDay,
  //     })
  //     .expect(HttpStatus.OK);

  //   expect(response.body.startDate).toEqual(
  //     updatedAvailability.startDate.toISOString(),
  //   );
  //   expect(response.body.endDate).toEqual(
  //     updatedAvailability.endDate.toISOString(),
  //   );
  //   expect(response.body.weekDay).toEqual(updatedAvailability.weekDay);
  // });

  it('should delete availability', async () => {
    const newAvailability = await AvailabilityFactory.create(prisma);

    await request(app.getHttpServer())
      .delete(`/availability/${newAvailability.id}`)
      .expect(HttpStatus.OK);

    const deletedAvailability = await prisma.availability.findUnique({
      where: { id: newAvailability.id },
    });

    expect(deletedAvailability).toBeNull();
  });
});
