import { ApexOptions } from "ng-apexcharts";

type FinancialMetrics = {
  data: string,
  total_month: number,
}

type SessionRequestMetrics = {
  per_day: number,
  session_date: string,
  percentage_increase: number,
  number_sessions_last_30_days: string
  number_sessions_last_60_days: string
}

type UserMetrics = {
  per_day: number,
  session_date: string,
  percentage_increase: number,
  number_sessions_last_30_days: string
  number_sessions_last_60_days: string
}

type SessionMetrics = {
  per_day: number,
  session_date: string,
  percentage_increase: number,
  number_sessions_last_30_days: string
  number_sessions_last_60_days: string
}

export const sessionChartOptions = (sessionMetrics: SessionMetrics[]): ApexOptions  => {

  const generalMetric = sessionMetrics[sessionMetrics.length - 1]
  const percentageIncrease = generalMetric.percentage_increase
  const color = percentageIncrease > 0 ? 'green' : (percentageIncrease < 0 ? 'red' : 'gray');
  const subtitle = percentageIncrease > 0 ? `⬆ ${percentageIncrease}%` : (percentageIncrease < 0 ? `⬇ ${percentageIncrease}%` : '0%')

  const data = sessionMetrics.map((d: SessionMetrics) => {
    return {
      x: d.session_date,
      y: Number(d.per_day)
    }
  })

  return  {
    chart: {
      id: 'sessionChart',
      // group: 'sparklines',
      type: 'area',
      height: 160,
      sparkline: {
        enabled: true
      },
    },
    stroke: {
      curve: 'smooth'
    },
    fill: {
      opacity: 1,
      colors: ['#734CEA','#F44336', '#E91E63', '#9C27B0']
    },
    series: [{
      name: 'Sessões',
      data
    }],
    labels: sessionMetrics.map((d: SessionMetrics) => d.session_date),
    yaxis: {
      min: 0
    },
    xaxis: {
      // type: 'datetime',
      // categories: ['05/06/2014', '05/08/2014', '05/28/2014'],
    },
    colors: ['#734CEA'],
    title: {
      text: `Sessões: ${generalMetric.number_sessions_last_30_days}`,
      offsetX: 30,
      style: {
        fontSize: '24px',
      }
    },
    subtitle: {
      text: subtitle,
      offsetX: 30,
      style: {
        fontSize: '14px',
        color,
        fontWeight: 'bold',
      }
    }
  }
}

export const usersChartOptions = (userMetrics: UserMetrics[]): ApexOptions => {

  const generalMetric = userMetrics[userMetrics.length - 1]
  const percentageIncrease = generalMetric.percentage_increase
  const color = percentageIncrease > 0 ? 'green' : (percentageIncrease < 0 ? 'red' : 'gray');
  const subtitle = percentageIncrease > 0 ? `⬆ ${percentageIncrease}%` : (percentageIncrease < 0 ? `⬇ ${percentageIncrease}%` : '0%')

  const data = userMetrics.map((d: UserMetrics) => {
    return {
      x: d.session_date,
      y: Number(d.per_day)
    }
  })

  return {
    chart: {
      id: 'usersChart',
      // group: 'sparklines',
      type: 'area',
      height: 160,
      sparkline: {
        enabled: true
      },
    },
    stroke: {
      curve: 'smooth',
      // colors: ['#F44336', '#E91E63', '#9C27B0']
    },
    fill: {
      type: "solid",
      opacity: 1,
      colors: ['#F44336', '#E91E63', '#9C27B0']
    },
    series: [{
      name: 'Aprendizes',
      data
    }],
    labels: userMetrics.map((d: SessionMetrics) => d.session_date),
    yaxis: {
      min: 0
    },
    xaxis: {
      // type: 'datetime',
    },
    colors: ["#FEB019"],
    title: {
      text: `Aprendizes: ${generalMetric.number_sessions_last_30_days}`,
      offsetX: 30,
      style: {
        fontSize: '24px',
      }
    },
    subtitle: {
      text: subtitle,
      offsetX: 30,
      style: {
        color,
        fontSize: '14px',
        fontWeight: 'bold',
      }
    }
  }
}

export const sessionRequestsChartOptions = (sessionRequestMetrics: SessionRequestMetrics[]): ApexOptions => {
  const generalMetric = sessionRequestMetrics[sessionRequestMetrics.length - 1]
  const percentageIncrease = generalMetric.percentage_increase
  const color = percentageIncrease > 0 ? 'green' : (percentageIncrease < 0 ? 'red' : 'gray');
  const subtitle = percentageIncrease > 0 ? `⬆ ${percentageIncrease}%` : (percentageIncrease < 0 ? `⬇ ${percentageIncrease}%` : '0%')

  const data = sessionRequestMetrics.map((d: SessionRequestMetrics) => {
    return {
      x: d.session_date,
      y: Number(d.per_day)
    }
  })

  return {
    chart: {
      id: 'sessionRequestsChart',
      // group: 'sparklines',
      type: 'area',
      height: 160,
      sparkline: {
        enabled: true
      },
    },
    stroke: {
      curve: 'smooth'
    },
    fill: {
      opacity: 1,
    },
    series: [{
      name: 'Solicitações',
      data
    }],
    labels: sessionRequestMetrics.map((d: SessionMetrics) => d.session_date),
    yaxis: {
      min: 0
    },
    xaxis: {
      // type: 'datetime',
    },
    colors: ["#00E396",],
    title: {
      text: `Solicitações: ${generalMetric.number_sessions_last_30_days}`,
      offsetX: 30,
      style: {
        fontSize: '24px',
      }
    },
    subtitle: {
      text: subtitle,
      offsetX: 30,
      style: {
        color,
        fontSize: '14px',
        fontWeight: 'bold',
      }
    }
  }
}

export const financialChartOptions = (financialMetrics: FinancialMetrics[]): ApexOptions => {

  const [last_year, actual_year] = [
    financialMetrics.slice(0, 12).map((d: FinancialMetrics) => d.total_month),
    financialMetrics.slice(12, 24).map((d: FinancialMetrics) => d.total_month)
  ];

  const max = Math.max(...financialMetrics.map((d: FinancialMetrics) => d.total_month))
  const min = Math.min(...financialMetrics.map((d: FinancialMetrics) => d.total_month))

  return {
    series: [
      {
        name: "Ano Atual",
        data: actual_year
      },
      {
        name: "Ano Anterior",
        data: last_year
      }
    ],
    chart: {
      height: 350,
      type: "line",
      dropShadow: {
        enabled: true,
        color: "#000",
        top: 18,
        left: 7,
        blur: 10,
        opacity: 0.2
      },
      toolbar: {
        show: true,
        offsetX: 0,
        offsetY: 0,
        tools: {
          download: true,
          selection: true,
          zoom: true,
          zoomin: true,
          zoomout: true,
          pan: true,
          customIcons: []
        },
        export: {
          csv: {
            filename: 'faturamento.csv',
            columnDelimiter: ',',
            headerCategory: 'category',
            headerValue: 'value',
            dateFormatter(timestamp: any) {
              return new Date(timestamp).toDateString()
            }
          },
          svg: {
            filename: 'faturamento.svg',
          },
          png: {
            filename: `faturamento.png`,
          }
        },
        autoSelected: 'zoom'
      }
    },
    colors: ["#77B6EA", "#545454"],
    dataLabels: {
      enabled: true,
      formatter: function(val) {
        return `R$ ${Number(val).toFixed(2)}`;
      }
    },
    stroke: {
      curve: "smooth"
    },
    title: {
      text: "Faturamento",
      align: "left"
    },
    grid: {
      borderColor: "#e7e7e7",
      row: {
        colors: ["#f3f3f3", "transparent"], // takes an array which will be repeated on columns
        opacity: 0.5
      }
    },
    markers: {
      size: 1
    },
    xaxis: {
      categories: ["Jan", "Fev", "Mar", "Abr", "Maio", "Jun", "Jul", "Ago", "Set", "Out", "Nov", "Dez"],
      title: {
        text: "Mês"
      }
    },
    yaxis: {
      title: {
        text: "Real (R$)"
      },
      min,
      max,
      labels: {
        show: true,
        formatter: function(val) {
          return `R$ ${Number(val).toFixed(2)}`;
        }
      }
    },
    legend: {
      position: "top",
      horizontalAlign: "right",
      floating: true,
      offsetY: -10,
      offsetX: -5
    },

  }
};
