import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { AuthenticationService } from 'src/app/services/auth/authentication.service';
import { ErrorService } from 'src/app/services/error.service';
import { UserService } from 'src/app/services/user.service';
import Swal from 'sweetalert2'
const swal = Swal as any;

@Component({
  selector: 'app-experiencias',
  templateUrl: './experiencias.component.html',
  styleUrls: ['./experiencias.component.scss']
})
export class ExperienciasComponent implements OnInit {

  user: any;
  experiences: any;

  experienciaForm: FormGroup;

  minDate = '1900-01-01';
  maxDate = new Date().toISOString().split('T')[0];

  constructor(
    public authenticationService: AuthenticationService,
    private router: Router,
    private userService: UserService,
    private errorService: ErrorService,
    private fb: FormBuilder,
  ) {
    this.experienciaForm = this.fb.group({
      id: [''],
      cargo: ['', Validators.required],
      empresa: ['', Validators.required],
      dataInicio: ['', Validators.required],
      dataFim: [''],
      descricao: ['', Validators.required],
    });
  }

  ngOnInit() {
    this.authenticationService.user$.subscribe(user => {
      this.user = user;
    })

    this.buscarExperiencias()
  }

  buscarExperiencias() {
    this.resetForm()
    this.userService.getUserExperiences(this.user.id).subscribe(data => {
      this.experiences = data;
    })
  }

  setEdit(experience: any) {
    this.experienciaForm.patchValue({
      id: experience.id,
      cargo: experience.role,
      empresa: experience.company,
      dataInicio: experience.startDate,
      dataFim: experience.endDate,
      descricao: experience.description,
    })
  }

  salvarExperiencia() {
    console.log('this.experienciaForm.value', this.experienciaForm.value)
    if (!this.experienciaForm.valid) {
      swal.fire('', 'Por favor preencha todos os campos', 'warning')
      return;
    }
    let model = {
      userId: this.user.id,
      company: this.experienciaForm.value['empresa'],
      role: this.experienciaForm.value['cargo'],
      description: this.experienciaForm.value['descricao'],
      startDate: new Date(this.experienciaForm.value['dataInicio']).toISOString(),
      endDate: this.experienciaForm.value['dataFim'] ? new Date(this.experienciaForm.value['dataFim']).toISOString() : undefined,
    }

    if (!model['endDate']) {
      delete model.endDate;
    }


    if (this.experienciaForm.value['id']) {
      this.userService.putUserExperience(this.user.id, this.experienciaForm.value['id'], model).subscribe(data => {
        swal.fire('', 'Experiência salva com sucesso', 'success')
        this.buscarExperiencias()
      }, error => {
        this.errorService.showErrorMessage(error)
      })

    } else {
      this.userService.postUserExperience(this.user.id, model).subscribe(data => {
        swal.fire('', 'Experiência cadastrada com sucesso', 'success')
        this.buscarExperiencias()
      }, error => {
        this.errorService.showErrorMessage(error)
      })
    }
  }

  excluirExperiencia(experience: any) {
    swal.fire({
      title: 'Deseja excluir a experiência?',
      showCancelButton: true,
      confirmButtonText: 'Confirmar exclusão',
      confirmButtonColor: '#dc2626',
      cancelButtonText: 'Cancelar',
      cancelButtonColor: '#374151',
      reverseButtons: true,
    }).then((result: any) => {
      if (result.isConfirmed) {
        this.userService.deleteUserExperience(this.user.id, experience.id).subscribe(data => {
          this.buscarExperiencias()
        }, error => {
          this.errorService.showErrorMessage(error)
        })
      }
    })
  }

  validarData(event: any) {
    const input = event.target as HTMLInputElement;
    const dataSelecionada = input.value;

    if (!dataSelecionada) {
      return;
    }
    
    if (dataSelecionada < this.minDate) {
      input.value = this.minDate;
    } else if (dataSelecionada > this.maxDate) {
      input.value = this.maxDate;
    }
  }

  resetForm() {
    this.experienciaForm.reset()
  }
}
