import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import { createMask } from '@ngneat/input-mask';
import { AuthenticationService } from 'src/app/services/auth/authentication.service';
import { ErrorService } from 'src/app/services/error.service';
import { PaymentService } from 'src/app/services/payment.service';
import { SessionService } from 'src/app/services/session.service';
import Swal from 'sweetalert2'
const swal = Swal as any;

@Component({
  selector: 'app-pagamento',
  templateUrl: './pagamento.component.html',
  styleUrls: ['./pagamento.component.scss']
})
export class PagamentoComponent implements OnInit {

  user: any;
  userCards: any;
  cardForm: FormGroup;
  flipCard: boolean = false;
  selectedCard: any;
  cadastrarCartao = false;
  mentor: any;
  totalPrice: any;

  detalhes: any;

  anoVencimentoSelectOptions: number[] = [];

  atributes: any = {
    background: '#1d4ed8',
    height: '200px',
    width: '300px',
    name: '',
    number: '',
    ccv: '',
    expiration: '',
  }

  cardNumberMask = createMask('9999 9999 9999 9999');
  cvvMask = createMask('999');

  pagamentoConcluido = false;
  linkCalendario: any;

  constructor(
    private authenticationService: AuthenticationService,
    private fb: FormBuilder,
    private sessionService: SessionService,
    private paymentService: PaymentService,
    private router: Router,
    private errorService: ErrorService,
  ) {
    this.cardForm = this.fb.group({
      numero: [''],
      nome: [''],
      validadeMes: [''],
      validadeAno: [''],
      cvv: [''],
      tipoCartao: ['']
    });

    const currentYear = new Date().getFullYear();
    for (let i = 0; i < 11; i++) {
      this.anoVencimentoSelectOptions.push(currentYear + i);
    }
  }

  ngOnInit() {
    this.authenticationService.user$.subscribe(user => {
      this.user = user;
    })

    if (!this.sessionService.sessionToPay) {
      this.router.navigate(['/mentores'])
      return;
    }

    this.detalhes = this.sessionService.sessionToPayDetails;
    this.detalhes['sessionDuration'] = this.converterDecimalParaHorasPorExtenso(this.detalhes['sessionDuration'])

    this.totalPrice = this.sessionService.sessionToPay['totalValue']

    this.paymentService.getPaymentByUser(this.user.id).subscribe(data => {
      this.userCards = data;
      if (this.userCards.length < 1) {
        this.cadastrarCartao = true;
      }
    })

    this.cardForm.valueChanges.subscribe(data => {
      this.atributes = {
        ...this.atributes,
        name: this.cardForm.value['nome'],
        number: this.cardForm.value['numero'],
        ccv: '999',
        expiration: '99/99',
      }
    })
  }

  onNovoCartao() {
    this.cadastrarCartao = true;
    this.selectedCard = ''
  }

  onChangeCard(id: any) {
    this.selectedCard = id;
    this.cadastrarCartao = false;
  }

  concluir() {
    let model = this.sessionService.sessionToPay;

    if (this.cadastrarCartao) {
      if (!this.validarDadosCartao(this.cardForm.value)) {
        return;
      }

      let cardModel: any = {
        number: this.cardForm.value['numero'].replaceAll(' ', ''),
        cvv: this.cardForm.value['cvv'],
        dueDate: this.cardForm.value['validadeMes'] + '/' + this.cardForm.value['validadeAno'],
        name: this.cardForm.value['nome'],
        userId: this.user.id,
        type: this.cardForm.value['tipoCartao']
      };
      this.paymentService.postPaymentCard(cardModel).subscribe(data => {
        model['paymentCardId'] = data.id;

        this.gravarSessao(model)
      }, error => {
        this.errorService.showErrorMessage(error)
      })
    } else {
      if (!this.selectedCard) {
        swal.fire('', 'Selecione um cartão', 'warning')
        return;
      }

      model['paymentCardId'] = this.selectedCard;

      this.gravarSessao(model)
    }
  }

  gravarSessao(model: any) {
    this.sessionService.postSession(model).subscribe(data => {
      if (!model['idCombinar']) {
        this.linkCalendario = this.createGoogleCalendarLink(model['startDate'], model['endDate'])
      }
      this.pagamentoConcluido = true;
    }, error => {
      this.errorService.showErrorMessage(error)
    })
  }

  validarDadosCartao(dadosCartao: any) {
    if (!dadosCartao.tipoCartao) {
      swal.fire('', 'Por favor selecione o tipo de cartão', 'warning')
      return false;
    }

    if (!dadosCartao.numero || !dadosCartao.nome || !dadosCartao.validadeMes || !dadosCartao.validadeAno || !dadosCartao.cvv || !dadosCartao.tipoCartao) {
      swal.fire('', 'Por favor preencha todos os dados do cartão', 'warning')
      return false;
    }

    if (!/^\d{16}$/.test(dadosCartao.numero.replaceAll(' ', ''))) {
      swal.fire('', 'Por favor preencha um número de cartão válido', 'warning')
      return false;
    }

    const mesValidade = parseInt(dadosCartao.validadeMes, 10);
    if (isNaN(mesValidade) || mesValidade < 1 || mesValidade > 12) {
      swal.fire('', 'Por favor selecione um mês válido', 'warning')
      return false;
    }

    if (!/^\d{4}$/.test(dadosCartao.validadeAno)) {
      swal.fire('', 'Por favor selecione um ano válido', 'warning')
      return false;
    }

    if (!/^\d{3}$/.test(dadosCartao.cvv)) {
      swal.fire('', 'Por favor preencha um CVV válido', 'warning')
      return false;
    }

    return true;
  }

  converterDecimalParaHorasPorExtenso(decimal: number): string {
    const horasInteiras = Math.floor(decimal);
    const minutosDecimais = (decimal - horasInteiras) * 60;
    const minutosInteiros = Math.round(minutosDecimais);

    let resultado = '';

    if (horasInteiras > 0) {
      resultado += `${horasInteiras} ${horasInteiras === 1 ? 'hora' : 'horas'}`;
    }

    if (minutosInteiros > 0) {
      if (resultado !== '') {
        resultado += ' e ';
      }

      resultado += `${minutosInteiros} ${minutosInteiros === 1 ? 'minuto' : 'minutos'}`;
    }

    if (resultado === '') {
      resultado = '0 minutos';
    }

    return resultado;
  }

  createGoogleCalendarLink(startDate: any, endDate: any) {
    const mentorName = this.detalhes.mentor.name.split(' ').join('+');
    const eventText = `[Mentora]+Sessão+com+${mentorName}`;
    const mentorEmail = this.detalhes.mentor.email;
    const eventDescription = '';
    const url = 'https://calendar.google.com/calendar/u/0/r/eventedit?';
    const queryParams = `text=${eventText}&add=${mentorEmail}&dates=${startDate}/${endDate}&details=${eventDescription}&location=&sf=true&output=xml#eventpage_6`

    return `${url}${queryParams}`;
  }
}